use std::collections::HashSet;

use serde::Deserialize;

use config::{Config, Environment};

#[derive(Debug, Deserialize, Clone)]
pub struct AppConfig {
    pub cache_dir: String,
    pub read_tokens: Option<HashSet<String>>,
    pub read_write_tokens: HashSet<String>,
    pub bind_addresses: Option<Vec<String>>,
}

pub fn load_config() -> AppConfig {
    let settings = Config::builder()
        .add_source(
            Environment::with_prefix("ENEX")
                .try_parsing(true)
                .list_separator(",")
                .with_list_parse_key("read_tokens")
                .with_list_parse_key("read_write_tokens")
                .with_list_parse_key("bind_addresses"),
        )
        .build()
        .expect("Could not initialize config.");
    settings
        .try_deserialize::<AppConfig>()
        .expect("Could not parse config.")
}
